/***************************************************************************//**
 *   @file   AD7298.c
 *   @brief  Implementation of AD7298 Driver.
 *   @author DBogdan (dragos.bogdan@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 751
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "AD7298.h"
#include "Communication.h"
#include "TIME.h"

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned short controlRegVal = 0;

/******************************************************************************/
/************************ Functions Definitions *******************************/
/******************************************************************************/

/***************************************************************************//**
 * @brief Initializes the SPI communication peripheral.
 *
 * @return status - The result of the I2C initialization.
 *                  Example: 0 - Initialization failed;
 *                           1 - Initialization succeeded.
*******************************************************************************/
unsigned char AD7298_Init(void)
{
    unsigned char status = 0;
    
    AD7298_PD_RST_SET_HIGH;
    AD7298_PD_RST_PIN_OUT;
    AD7298_TSENSE_BSY_PIN_IN;
    status = SPI_Init(0, 1000000, 1, 1);
    
    return status;
}

/***************************************************************************//**
 * @brief Powers up or powers down the device. When powering up user must 
 *        configure the control register before attempting to read a conversion 
 *        data.
 * @param powerOption - Device power mode option.
 *                      Example: AD7298_PWR_POWER_UP
 *                               AD7298_PWR_PARTIAL_POWER_DOWN
 *                               AD7298_PWR_FULL_POWER_DOWN
 *
 * @return None.
*******************************************************************************/
void AD7298_SetPower(unsigned char powerOption)
{
    switch(powerOption)
    {
        case  AD7298_PWR_POWER_UP:
            if(AD7298_PD_RST_STATE == 0)
            {
                AD7298_PD_RST_SET_HIGH;
                TIME_DelayMs(6);
                TIME_DelayUs(200);
            }
            else
            {
                controlRegVal &= ~AD7298_PPD;               // Clear PPD bit
                AD7298_SetControlRegister(controlRegVal);
            }
            break;
            
        case AD7298_PWR_PARTIAL_POWER_DOWN:
            AD7298_SetControlRegister(controlRegVal | AD7298_PPD);    
            break;
            
        case AD7298_PWR_FULL_POWER_DOWN:
            AD7298_PD_RST_SET_LOW;
            break;    
    }
}

/***************************************************************************//**
 * @brief Sets the Control Register.
 *
 * @param value - The value to write in the Control Register.
 *
 * @return None.
*******************************************************************************/
void AD7298_SetControlRegister(unsigned short value)
{
    unsigned char sendBuffer[2] = {0, 0};
        
    sendBuffer[0] = (value & 0xFF00) >> 8;
    sendBuffer[1] = (value & 0x00FF);
    SPI_Write(AD7298_SLAVE_ID, sendBuffer, 2);
    /* Store the last value written to Control Register. */
    controlRegVal = value;
}

/***************************************************************************//**
 * @brief Configures the conversion settings.
 *
 * @param repeat - enable repeated conversion of the selected sequence of
 *                 channels.
 *                 Example: 0 - disable repeated conversion;
 *                          1 - enable repeated conversion.
 * @param channels - selected channels.
 *                   Example: AD7298_CH(0) - channel 0;
 *                            ...
 *                            AD7298_CH(7) - channel 7.
 * @param extRef - enable the use of an external reference.
 *                 Example: 0 - disable the external reference;
 *                          1 - enable the external reference.
 * @param tSense - enable the temperature conversion.
 *                 Example: 0 - disable the temperature conversion;
 *                          1 - enable the temperature conversion.
 * @param tSenseAvg - enable the temperature sensor averaging function.
 *                    Example: 0 - disable the temp. sensor averaging function;
 *                             1 - enable the temp. sensor averaging function.
 *
 * @return None.
*******************************************************************************/
void AD7298_ConfigConversion(unsigned char repeat,
                             unsigned short channels,
                             unsigned char extRef,
                             unsigned char tSense,
                             unsigned char tSenseAvg)
{
    unsigned short controlReg = 0;
    
    controlReg = AD7298_WRITE |
                 (repeat * AD7298_REPEAT) |
                 channels |
                 (extRef * AD7298_EXTREF) |
                 (tSense * AD7298_TSENSE) |
                 (tSenseAvg * AD7298_TAVG);
    AD7298_SetControlRegister(controlReg);
}

/***************************************************************************//**
 * @brief Returns the conversion value and its corresponding channel index.
 *        
 * @param convData - Stores the 12-bit conversion data.
 * @param ch - Stores the channel index.
 *
 * @return None.
*******************************************************************************/
void AD7298_GetConversionResult(unsigned short *convData, unsigned char *ch)
{
    unsigned char  receiveBuffer[2] = {0, 0};
    unsigned short value = 0;
    
    SPI_Read(AD7298_SLAVE_ID, receiveBuffer, 2);
    value = ((unsigned short)receiveBuffer[0] << 8) + receiveBuffer[1];
    *convData = value & 0x0FFF;
    *ch = (value & 0xF000) >> 12;
}

/**************************************************************************//**
 * @brief Calculates the temperature in degrees Celsius.
 *
 * @param value - Temperature raw value.
 *
 * @return Degrees Celsius.
******************************************************************************/
float AD7298_CalcTemp(unsigned short value, float vRef)
{
    float degCelsius = 0;
    
    if(value >= 0x800)
    {
        value = 4096 - value; 
        value *= -1;
    }
    degCelsius = vRef * (((float)value / 10) + 109.3) - 273.15;
    
    return degCelsius;
}

/**************************************************************************//**
 * @brief Calculates the measured voltage.
 *
 * @param value - Raw value of the analog-to-digital conversion.
 *
 * @return Volts.
******************************************************************************/
float AD7298_CalcVoltage(unsigned short value, float vRef)
{
   float volts = 0;
   
   volts = ((float)value * vRef) / 4096;

   return volts;
}
